include ${top_srcdir}/config.mk

# wild guess (target for default $CC compiler)
BUILD?=$(shell $(CC) -dumpmachine)

# predefined hosts for target binaries
ifeq ($(SYS),native)
HOST?=$(BUILD)
endif
ifeq ($(SYS),mingw64)
HOST?=x86_64-w64-mingw32
endif
ifeq ($(SYS),mingw32)
HOST?=i686-pc-mingw32
endif

BUILDDIR=$(NAME)-$(SYS)-build
DESTDIR=$(NAME)-$(SYS)-dest
GET_VERSION=`cat $(top_srcdir)/$(DISTDIR)/$(NAME)-version.txt`
DISTFILE=$(top_srcdir)/$(DISTDIR)/$(SYS)/$(NAME)-$(GET_VERSION)-$(SYS).zip

#pkg-config and cross builds
# 1) force using native pkg-config, Fedora screwed mingw's pkg-config
# 2) PKG_CONFIG_LIBDIR is different for cross-compiling,
#    we will specify the system directory ourself in PKG_CONFIG_PATH
ifneq ($(SYS),native)
CONFIGURE_CROSS_ENV=PKG_CONFIG_LIBDIR= PKG_CONFIG=pkg-config
endif
CONFIGURE_ARGS?=
CONFIGURE_ARGS_OVERRIDE?=--prefix=$(PREFIX) --host=$(HOST) --build=$(BUILD)

FETCH_METHOD?=$(shell echo "$(FETCH_URL)" | cut -f1 -d:)
CMD_EXTRACT=$(shell set -o xtrace; if echo "$(FETCH_URL)" | grep '.tar.gz$$\|.tgz$$' >/dev/null; then \
	echo 'tar xzvf'; \
elif echo "$(FETCH_URL)" | grep '.tar.bz2$$\|.tbz$$' >/dev/null; then \
	echo 'tar xjvf'; \
elif echo "$(FETCH_URL)" | grep '.zip$$' >/dev/null; then \
	echo 'unzip'; \
fi)

ifeq ($(FETCH_METHOD),svn)
CMD_CO=svn co "$(FETCH_URL)" $(NAME)
CMD_UP=svn up
endif
ifeq ($(FETCH_METHOD),hg)
CMD_CO=hg clone "$(FETCH_URL)" $(NAME)
CMD_UP=hg fetch -r tip
endif
ifeq ($(FETCH_METHOD),cvs)
CMD_CO=cvs -z1 -d"$(CVSROOT)" co $(CVSMODULE); if test "x$(CVSMODULE)" != "x$(NAME)"; then mv $(CVSMODULE) $(NAME); fi
CMD_UP=cvs -z1 update -AdP
endif
ifeq ($(FETCH_METHOD),git)
CMD_CO=git clone "$(FETCH_URL)" $(NAME)
CMD_UP=git pull
endif
ifeq ($(FETCH_METHOD),ftp)
FETCH_METHOD:=http
endif
ifeq ($(FETCH_METHOD),http)
FILE=$(shell basename $(FETCH_URL))
CMD_CO=if test -f "$(top_srcdir)/$(DISTDIR)/$(FILE)"; then cp "$(top_srcdir)/$(DISTDIR)/$(FILE)" .; else wget -c "$(FETCH_URL)"; fi; $(CMD_EXTRACT) "$(FILE)"; mv "$(FETCH_DIR)" "$(NAME)"
CMD_UP=
endif
ifeq ($(FETCH_METHOD),)
CMD_CO=mkdir "$(NAME)"
CMD_UP=
endif

main-fetch .$(NAME)-fetched:
	rm -rf ./$(NAME) $(top_srcdir)/$(DISTDIR)/$(NAME)-version.txt $(top_srcdir)/$(DISTDIR)/$(NAME)-*-src.tar.gz .$(NAME)*
	$(CMD_CO)
ifeq ($(FETCH_METHOD),http)
	mv "$(FILE)" $(top_srcdir)/$(DISTDIR)/
endif
	$(MAKE) -C $(NAME) -f $(top_srcdir)/../packages/$(NAME)/Makefile top_srcdir=$(top_srcdir)/.. version
ifneq ($(FETCH_METHOD),http)
	tar cf - $(NAME) | gzip --fast > $(top_srcdir)/$(DISTDIR)/$(NAME)-$(GET_VERSION)-src.tar.gz
endif
	touch .$(NAME)-fetched

main-update:
	rm -rf ./$(NAME) .$(NAME)*
ifeq ($(FETCH_METHOD),http)
	$(CMD_EXTRACT) "$(top_srcdir)/$(DISTDIR)/$(FILE)"
	mv "$(FETCH_DIR)" "$(NAME)"
else
	gzip -dc $(top_srcdir)/$(DISTDIR)/$(NAME)-$(GET_VERSION)-src.tar.gz | tar xf -
endif
	cd $(NAME); $(CMD_UP)
	$(MAKE) -C $(NAME) -f $(top_srcdir)/../packages/$(NAME)/Makefile top_srcdir=$(top_srcdir)/.. version
	tar cf - $(NAME) | gzip --fast > $(top_srcdir)/$(DISTDIR)/$(NAME)-$(GET_VERSION)-src.tar.gz
	touch .$(NAME)-fetched

main-patch .$(NAME)-patched: .$(NAME)-fetched
	if ! test -f .$(NAME)-patched; then \
		ls -1 $(top_srcdir)/packages/$(NAME)/*.patch $(top_srcdir)/packages/$(NAME)/*.diff 2>/dev/null | while read p; do \
			(cd $(NAME); patch -p1 < ../$$p); \
		done; \
		touch .$(NAME)-patched; \
	fi

main-autogen .$(NAME)-auto: .$(NAME)-patched
ifneq ($(USE_AUTOGEN),)
	cd $(NAME); $(AUTOGEN_ENV) $(USE_AUTOGEN) $(AUTOGEN_ARGS)
endif
	touch .$(NAME)-auto

main-configure .$(NAME)-conf-$(SYS): .$(NAME)-auto
	rm -rf $(BUILDDIR)
	mkdir -p $(BUILDDIR)
ifeq ($(NO_CONFIGURE),)
	cd $(BUILDDIR); \
	$(CONFIGURE_ENV) \
	$(CONFIGURE_CROSS_ENV) \
	PKG_CONFIG_PATH=$(PREFIX)/lib/pkgconfig:$(PKG_CONFIG_PATH) \
	LD_LIBRARY_PATH=$(PREFIX)/lib:$$LD_LIBRARY_PATH \
	../$(NAME)/configure $(CONFIGURE_ARGS_OVERRIDE) $(CONFIGURE_ARGS)
endif
	touch .$(NAME)-conf-$(SYS)

main-build .$(NAME)-built-$(SYS): .$(NAME)-conf-$(SYS)
	$(MAKE) -f $(top_srcdir)/packages/$(NAME)/Makefile MAKEFLAGS=$(space) build
	touch .$(NAME)-built-$(SYS)

main-install .$(NAME)-installed-$(SYS): .$(NAME)-built-$(SYS)
	rm -rf $(DESTDIR) /tmp/$${pkg}-root; \
	$(MAKE) -f $(top_srcdir)/packages/$(NAME)/Makefile MAKEFLAGS="DESTDIR=/tmp/$(NAME)-root" install
	mv /tmp/$(NAME)-root $(DESTDIR); \
	touch .$(NAME)-installed-$(SYS)

main-dist .$(NAME)-dist-$(SYS): .$(NAME)-installed-$(SYS)
	rm -fv $(DISTFILE)
	cd $(DESTDIR); zip -r ../$(top_srcdir)/$(DISTDIR)/$(SYS)/$(NAME)-`cat ../$(top_srcdir)/$(DISTDIR)/$(NAME)-version.txt`-$(SYS).zip .
	touch .$(NAME)-dist-$(SYS)
	#tar cf - -C $(DESTDIR) . | gzip --fast > $(DISTFILE)

main-stage .$(NAME)-staged-$(SYS): .$(NAME)-dist-$(SYS)
	if ! test -f .$(NAME)-staged-$(SYS); then \
		mkdir -p "$(PREFIX)" || /bin/true; \
		unzip -d / $(DISTFILE); \
		touch .$(NAME)-staged-$(SYS); \
	fi

main-meta:
	rm -rf $(PREFIX)-metapackageroot
	mkdir $(PREFIX)-metapackageroot
	cat $(top_srcdir)/packages/$(METAPACKAGE)-$(SYS).list | while read line; do \
		echo "read: $$line"; \
		if echo "$$line" | grep ^@ >/dev/null; then \
			NAME=`echo $$line | sed 's/^.//'`; \
			$(MAKE) -C $(top_srcdir) $$NAME; \
			unzip -d $(PREFIX)-metapackageroot $(top_srcdir)/$(DISTDIR)/$(SYS)/$$NAME-`cat $(top_srcdir)/$(DISTDIR)/$$NAME-version.txt`-$(SYS).zip; \
		else \
			mkdir -p $(PREFIX)-metapackageroot$(PREFIX)/`dirname $$line`; \
			if test -f $(PREFIX)/$$line; then \
				cp $(PREFIX)/$$line $(PREFIX)-metapackageroot$(PREFIX)/$$line; \
			elif test -n "$(SYS_PREFIX)" -a -f "$(SYS_PREFIX)/$$line"; then \
				cp $(SYS_PREFIX)/$$line $(PREFIX)-metapackageroot$(PREFIX)/$$line; \
			elif test -n "$(SYS_PREFIX2)" -a -f "$(SYS_PREFIX2)/$$line"; then \
				cp $(SYS_PREFIX2)/$$line $(PREFIX)-metapackageroot$(PREFIX)/$$line; \
			else \
				echo File "$$line" not found.; \
			fi; \
		fi; \
	done
	if test "x$(STRIP)" != "x"; then \
		find $(PREFIX)-metapackageroot$(PREFIX)/bin -type f | \
			xargs -r -L32 $(HOST)-strip; \
		find $(PREFIX)-metapackageroot$(PREFIX)/lib -type f -iname *.a | \
			xargs -r -L32 $(HOST)-strip; \
		find $(PREFIX)-metapackageroot$(PREFIX)/lib -type f -iname *.dll | \
			xargs -r -L32 $(HOST)-strip; \
		find $(PREFIX)-metapackageroot$(PREFIX)/lib -type f -iname *.so | \
			xargs -r -L32 $(HOST)-strip; \
	fi
	-mkdir $(top_srcdir)/$(DISTDIR)/$(SYS)/metapackages
	cd $(PREFIX)-metapackageroot; zip -r $(PREFIX)/$(METAPACKAGE)-$(SYS).zip .
	mv $(PREFIX)/$(METAPACKAGE)-$(SYS).zip $(top_srcdir)/$(DISTDIR)/$(SYS)/metapackages/

main-clean:
	rm -Rf $(BUILDDIR) $(DESTDIR)
	rm -rvf .$(NAME)-*-$(SYS)
